@@ -4,7 +4,7 @@
     XML files. XGrep is distributed under the terms of the GNU General 
     Public License, a copy of which is included in the distribution.
 
-    Copyright (C) 2003,2008 Brendt Wohlberg  <software@wohlberg.net>
+    Copyright © 2003-2013 Brendt Wohlberg  <software@wohlberg.net>
 
 ********************************COPYRIGHT*************************************/
 
@@ -15,6 +15,7 @@
 #include <ctype.h>
 #include <regex.h>
 #include <assert.h>
+#include <libxml/xpathInternals.h>
 
 #include "find.h"
 #include "treex.h"
@@ -104,7 +105,7 @@ char* NewCopyRegexSplitStr(const char* str, const char* regex,
   char *cp = 0, *newstr = 0;
 
   if (regcomp(&preg, regex, REG_EXTENDED) == 0) {
-    if ((newstr = malloc(strlen(str)+2))) {
+    if ((newstr = malloc(strlen(str)+3))) {
       strcpy(newstr, str);
       newstr[strlen(newstr)+1] = '\0';
       newstr[strlen(newstr)] = ' ';
@@ -272,7 +273,7 @@ void PrintSubtreeRegex(FILE* fp, SubtreeRegexPtr stre) {
 
 xmlNodeSetPtr SubtreeRegexNodeSet(xmlDocPtr doc, SubtreeRegexPtr stre) {
   xmlNodeSetPtr nodeset = 0;
-  
+
   if (doc) {
     if ((nodeset = malloc(sizeof(xmlNodeSet)))) {
       nodeset->nodeNr = 0;
@@ -291,6 +292,15 @@ void SubtreeRegexNodeSetRecursive(xmlNodePtr node, SubtreeRegexPtr stre,
   xmlNodePtr cur = 0;
 
   if (node && stre && nodeset) {
+
+    /* Enable this section to allow matching the document root element */
+#if 0
+    if (SubtreeRegexMatch(node, stre))
+	AppendToNodeSet(nodeset, node);
+      else
+	SubtreeRegexNodeSetRecursive(cur, stre, nodeset);
+#endif
+
     cur = xmlGetFirstChildElement(node);
     while (cur) {
       if (SubtreeRegexMatch(cur, stre))
@@ -420,7 +430,9 @@ int AppendToNodeSet(xmlNodeSetPtr nodeset, xmlNodePtr node) {
 }
 
 
-int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt) {
+int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt, xmlChar** nslst, int nns,
+		    char tflg)
+{
   xmlXPathContextPtr xpcntxt = 0;
   xmlXPathObjectPtr xpobj = 0;
   xmlNodeSetPtr nodeset = 0;
@@ -429,14 +441,29 @@ int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt) {
 
   if (doc && xpathtxt) {
     if ((xpcntxt = xmlXPathNewContext(doc))) {
+      
+      if (nslst && nns > 0) {
+	int k;
+	xmlChar* cp;
+	for (k = 0; k < nns; k++) {
+	  if ((cp = (xmlChar*)strchr((const char*)nslst[k], '='))) {
+	    *(cp++) = '\0';
+	    if (xmlXPathRegisterNs(xpcntxt, nslst[k], cp) != 0)
+	      return 2;
+	  }   
+	}
+      }
+
       if ((xpobj = xmlXPathEvalExpression(xpathtxt, xpcntxt))) {
 	nodeset = xpobj->nodesetval;
 	sprintf(note, "(XPath: %.500s)", xpathtxt);
-	PrintNodeSet(nodeset, note);
+	PrintNodeSet(doc, nodeset, note, tflg);
 	xmlXPathFreeObject(xpobj);
       } else
 	ret = 0;
+
       xmlXPathFreeContext(xpcntxt);
+
     } else
       ret = 0;
   }
@@ -445,7 +472,7 @@ int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt) {
 }
 
 
-int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt) {
+int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt, char tflg) {
   xmlNodeSetPtr nodeset = 0;
   SubtreeRegexPtr stre = 0;
   char note[512];
@@ -454,7 +481,7 @@ int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt) {
     return 0;
   nodeset = SubtreeRegexNodeSet(doc, stre);
   sprintf(note, "(Search: %.500s)", srchstr);
-  PrintNodeSet(nodeset, note);
+  PrintNodeSet(doc, nodeset, note, tflg);
   xmlXPathFreeNodeSet(nodeset);
   FreeSubtreeRegex(stre);
 
@@ -462,7 +489,7 @@ int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt) {
 }
 
 
-void PrintNodeSet(xmlNodeSetPtr nodeset, char* note) {
+void PrintNodeSet(xmlDocPtr doc, xmlNodeSetPtr nodeset, char* note, char tflg) {
   xmlNodePtr node = 0;
   xmlBufferPtr buffer = 0;
   xmlChar* c;
@@ -472,28 +499,52 @@ void PrintNodeSet(xmlNodeSetPtr nodeset, char* note) {
     
     if (!note)
       note = "";
-    
-    if (xmlXPathNodeSetGetLength(nodeset) > 0)
+
+    if (!tflg && xmlXPathNodeSetGetLength(nodeset) > 0)
       printf("<!--         Start of node set %s                 -->\n", note);
-    
+
     for (k = 0;k < xmlXPathNodeSetGetLength(nodeset); k++) {
       node = xmlXPathNodeSetItem(nodeset, k);
-      printf("<!--         Node %3d in node set               -->\n", k);
-      
-      if (node->prev && xmlNodeIsText(node->prev)) {
-	c = xmlNodeGetContent(node->prev);
-	printf("%s", c);
-	xmlFree(c);
-      }
-      
-      buffer = xmlBufferCreate();
-      xmlNodeDump(buffer, node->doc, node, 0, 1);
-      xmlBufferDump(stdout, buffer);
-      printf("\n\n");
-      xmlBufferFree(buffer);
+      if (!tflg)
+	printf("<!--         Node %3d in node set               -->\n", k);
+
+      /* Separate handling of XML_NAMESPACE_DECL nodes is required since 
+         they have non-NULL prev field containing an invalid node pointer, 
+         and cause a segfault when passed to xmlNodeDump. */
+      if (!tflg && node->type != XML_NAMESPACE_DECL && node->prev && 
+	  xmlNodeIsText(node->prev)) {
+	  c = xmlNodeGetContent(node->prev);
+	  printf("%s", c);
+	  xmlFree(c);
+	}
+
+	buffer = xmlBufferCreate();
+	xmlNodeDump(buffer, doc, node, 0, tflg?0:1);
+
+	if (tflg) {
+	  /* It would be more efficient to use strpbrk to deal with
+	     both characters in the same call, but unfortunately there
+	     is no xmlStrpbrk. */
+	  c = buffer->content;
+	  while ((c = (xmlChar*)xmlStrchr(c, '\n'))) {
+	    *c = ' ';
+	  }
+	  c = buffer->content;
+	  while ((c = (xmlChar*)xmlStrchr(c, '\r'))) {
+	    *c = ' ';
+	  }
+	}
+
+	xmlBufferDump(stdout, buffer);
+	xmlBufferFree(buffer);
+
+	printf("\n");
+	if (!tflg)
+	  printf("\n");
+
     }
-    
-    if (xmlXPathNodeSetGetLength(nodeset) > 0)
+
+    if (!tflg && xmlXPathNodeSetGetLength(nodeset) > 0)
       printf("<!--         End of node set                    -->\n");
   }
 }
