@@ -4,7 +4,7 @@
     XML files. XGrep is distributed under the terms of the GNU General 
     Public License, a copy of which is included in the distribution.
 
-    Copyright (C) 2003,2008 Brendt Wohlberg  <software@wohlberg.net>
+    Copyright © 2003-2013 Brendt Wohlberg  <software@wohlberg.net>
 
 ********************************COPYRIGHT*************************************/
 
@@ -86,13 +86,14 @@ int AppendToNodeSet(xmlNodeSetPtr nodeset, xmlNodePtr node);
 
 
 /* Print subtrees from doc selected by the XPath xpathtxt */
-int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt);
+int PrintXPathNodes(xmlDocPtr doc, xmlChar* xpathtxt, xmlChar** nslst, 
+		    int nns, char tflg);
 
 /* Print subtrees from doc selected by the search string srchstr */
-int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt);
+int PrintSearchStrNodes(xmlDocPtr doc, char* srchstr, RegexType rt, char tflg);
 
 /* Print nodes in nodeset */
-void PrintNodeSet(xmlNodeSetPtr nodeset, char* note);
+void PrintNodeSet(xmlDocPtr doc, xmlNodeSetPtr nodeset, char* note, char tflg);
 
 #endif
 
